/***********************************************************************/
/*                                                                     */
/*  FILE        : HD44780_lcd.c                                        */
/*  DATE        : 12.01.11                                             */
/*  DESCRIPTION : lcd control functions, based on Elektor 02/2005      */
/*  PIC18flash development PCB demonstration firmware                  */
/*  CPU TYPE    : PIC18F2423      									   */
/*  NAME   	    : K. Riedel                                            */
/*                                                                     */
/***********************************************************************/

#include <delays.h>

#include "hd44780_lcd.h"

// DATA_PORT defines the port to which the LCD data lines are connected
#define DATA_PORT      PORTB
#define TRIS_DATA_PORT TRISB

#define RW_PIN   PORTCbits.RC7   		// PORT for RW 
#define TRIS_RW  TRISCbits.TRISC7  		// TRIS for RW 
#define RS_PIN   PORTBbits.RB5   		// PORT for RS 
#define TRIS_RS  TRISBbits.TRISB5  		// TRIS for RS 
#define E_PIN    PORTBbits.RB4   		// PORT for E 
#define TRIS_E   TRISBbits.TRISB4  		// TRIS for E 

#define LN_INPUT  TRIS_DATA_PORT|=0x0f  // make low nibble input
#define LN_OUTPUT TRIS_DATA_PORT&=0xf0  // make low nibble output

#define DISPLAY_ON  0b00001111  
#define DISPLAY_OFF 0b00001011  
#define CURSOR_ON   0b00001111  
#define CURSOR_OFF  0b00001101  
#define BLINK_ON    0b00001111  
#define BLINK_OFF   0b00001110  


void delay_1us (void)  
{  
	Nop ();		// 1uS @4mhz
}

void delay_20ms (void)
{
	Delay1KTCYx (20); 
	return;
}

void delay_5ms (void)
{
	Delay1KTCYx (5); 
	return;
}

void SleepMS(int MilliSeconds)
{
	int i;
	for (i=0; i<MilliSeconds; i++)
	{
		Delay1KTCYx(1);									// 1 msec = 1000 cycles at 4Mhz/1 MIPS
	}
	return;
}
void clock_data (void)
{
   delay_1us ();                  		 // delay ~ 1 microsecond
   E_PIN = 1;                     		 // set LCD enable pin HI
   delay_1us ();                  		 // delay ~ 1 microsecond
   E_PIN = 0;                     		 // set LCD enable pin LO
}

unsigned char LCD_busy (void)
{
   unsigned char port_data, port_busypin;
 
   LN_INPUT; 							// make low nibble input
   RW_PIN = 1;                     		// set the control bits for read
   RS_PIN = 0;
   delay_1us ();
   E_PIN = 1;                      		// clock in the command
   delay_1us ();
   port_data = DATA_PORT;
   port_busypin = port_data & 0b00001000; // 0x08;
   E_PIN = 0;              				// reset clock line
   delay_1us ();
   E_PIN = 1;              				// clock in other nibble
   delay_1us ();
   E_PIN = 0;
   RW_PIN = 0;             				// reset control line
   LN_OUTPUT;  							// make low nibble output
   return port_busypin;              
}

void clock_nibble_out (unsigned char data) // to low nibble of port
{
   DATA_PORT &= 0xf0;  					// clear low nibble of port
   data      &= 0x0f;  					// zero any bits in upper nibble of data
   DATA_PORT |= data;  					// set data to port  
   delay_1us ();
   E_PIN = 1;                      		// clock the data out
   delay_1us ();
   E_PIN = 0;                                
}

void LCD_command (unsigned char cmd) 
{
   LN_OUTPUT;  // low nibble output
   RW_PIN = 0;                     		// set control signals for command
   RS_PIN = 0;
   clock_nibble_out (cmd >> 4); 		// shift upper nibble to lower nibble
   clock_nibble_out (cmd);      		// this outputs lower nibble
}

void LCD_initialise (void)
{
   // LCD will be used in 4-bit mode. 
   // clock_nibble_out uses lower nibble of data port
   // LCD defaults on power-up to 8-bit mode
   
   TRIS_RW = 0;                    		// set RW pin for output
   TRIS_RS = 0;                    		// set RS pin for output
   TRIS_E  = 0;                    		// set  E pin for output
   RW_PIN  = 0;                   	 	// R/W pin made low
   RS_PIN  = 0;                   	 	// register select pin made low
   E_PIN   = 0;                   	 	// clock pin made low

   delay_20ms (); 						// delay 20ms to allow for LCD power-on reset
        
   LN_OUTPUT;    						// set data port low nibble for output

   clock_nibble_out (0b0011);     		// "home"
   delay_5ms ();  						// delay for at least 4.1ms
   clock_nibble_out (0b0011);     		// "home"
   delay_5ms ();       					// delay for at least 100us

   clock_nibble_out (0b0011);       	// "home"
   delay_5ms ();       					// delay for at least 100us
   clock_nibble_out (0b0010);   		// set LCD interface to 4-bit
   delay_5ms ();

   clock_nibble_out (0b0010);   		// set LCD interface to 4-bit
   clock_nibble_out (0b1000);   		// 1 = 2-lines, 0 = 5x7, 00 = xx

   while (LCD_busy ());            		// wait if LCD busy
 
   LCD_command (0x01);             		// clear display - takes ~16 ms
   delay_20ms ();
   LCD_command (DISPLAY_ON & CURSOR_OFF & BLINK_OFF);   
}

void LCD_writechar (char data)
{
   LN_OUTPUT;  							// low nibble output
   RW_PIN = 0;                     		// set control signals for data
   RS_PIN = 1;							// select data register
   clock_nibble_out (data >> 4); 		// shift upper nibble to lower nibble
   clock_nibble_out (data);      		// this outputs lower nibble
   LN_INPUT;
}

void LCD_displayRAM (unsigned char line, unsigned char position, const char *text)
{
   int i = 0;
   unsigned char address;

   address = position - 1;
   if (line == 2) address += 0x40;
   address += 0b10000000;
   while (LCD_busy ()); 
   LCD_command (address);
   while (text[i])                  	// write data to LCD up to null
   {
      while (LCD_busy ());      		// wait while LCD is busy
      LCD_writechar (text[i]); 			// write character to LCD
      i++; 								// step to next character
   }
}

void LCD_displayROM (unsigned char line, unsigned char position, rom const char *text)
{
   int i = 0;
   unsigned char address;

   address = position - 1;
   if (line == 2) address += 0x40;
   address += 0b10000000;
   while (LCD_busy ()); 
   LCD_command (address);
   while (text[i])                  	// write data to LCD up to null
   {
      while (LCD_busy ());      		// wait while LCD is busy
      LCD_writechar (text[i]); 			// write character to LCD
      i++; 								// step to next character
   }
}

void LCD_blink_position (unsigned char line, unsigned char position)
{
   unsigned char address;

   address = position - 1;
   if (line == 2) address += 0x40;
   address += 0b10000000;
   while (LCD_busy ()); 
   LCD_command (address);
   while (LCD_busy ()); 
   LCD_command (DISPLAY_ON & CURSOR_ON & BLINK_ON);
}

void LCD_normal (void)
{
   while (LCD_busy ()); 
   LCD_command (DISPLAY_ON & CURSOR_OFF & BLINK_OFF);
}

void ltoa_format (char *erg, long zahl, unsigned int vk, unsigned int nk, char vorz)
// Out-String, input long, pre-decimal digits, decimal digits, sign '+' or ' '
{
   long temp;
   int  i;
   i = vk + nk + 1;                   	// string length 
   erg[i--] = 0;                      	// string end
   if ( zahl == 0 )						// special case input = 0  
   {
      while( i >= 0 && 
	       ( zahl > 0 || i+2*( nk != 0 ) > vk) ) // vk contain sign
	   	{
	      if (i==vk&&nk!=0) erg[i--]='.';  // decimal point, if nk is executed
	      else erg[i--] = '0'; // detach digit, value = 0
	  	}
	} 
   else 
   {
      if ( zahl < 0 )
      {
      vorz  = '-';                     // sign = '-'
      zahl *=  -1;                     // calculate further with positive value
      }
      while( i >= 0 && 
	       ( zahl > 0 || i+2*( nk != 0 ) > vk) ) // vk contain sign  
      {
      if (i==vk&&nk!=0) erg[i--]='.';  // decimal point, if nk is executed  
      else 
      {
      temp     =  zahl / 10;           	// integer division  
      erg[i--] = (zahl - temp*10) + 48; // detach digit, assign ASCII-value
      zahl     =  temp;                	// for next pass reduce digit
      }
   	  }
   if ( i >= 0 ) erg[i--] = vorz;      	// write sign
   }
   while( i >= 0 ) erg[i--] = ' ';     	// fill begin with spaces

}	