/***********************************************************************/
/*                                                                     */
/*  FILE        : main.c                                               */
/*  DATE        : 13.01.11                                             */
/*  DESCRIPTION : Counter Main Program                                 */
/*  CPU TYPE    : PIC18F2423      									   */
/*  NAME   	    : K. Riedel                                            */
/*                                                                     */
/***********************************************************************/

#include <p18f2423.h>
#include <adc.h>
#include <timers.h>
#include "hd44780_lcd.h"

#pragma config WDT = OFF
#pragma config OSC = HS
#pragma config FCMEN = OFF
#pragma config BOREN = OFF
#pragma config LVP = OFF
#pragma config MCLRE = ON
#pragma config PBADEN = OFF
#pragma config PWRT = ON

void high_priority_isr (void);
unsigned long result;
char buffer[24];

#pragma code high_vector=0x08

void high_interrupt (void)
{
  _asm 
  BCF 0xfd5, 0x7, ACCESS 	// Disable Timer0, direct programming in c is too slow
  BCF 0xfcd, 0, ACCESS  	// Disable Timer1
  GOTO high_priority_isr
  _endasm  					// Inline assembly that will jump to the high priority ISR.
}
#pragma code

#pragma interrupt high_priority_isr
void high_priority_isr (void)
{
		T1CONbits.TMR1ON = 0;  			// Disable Timer1
		PIE1bits.TMR1IE = 0;   			// Disable Timer1 overflow interrupts 
  		
  		result = ReadTimer0();
  					
		PORTCbits.RC4 = !PORTCbits.RC4;	// flash led
		
		if (PORTCbits.RC2)
		{
			result *= 64;					// Timer 0 Prescaler 64:1
			result /= 100;					// scale result -> LTC1799
			ltoa_format (buffer, result, 3, 3, ' '); // format display output LTC1799
			LCD_displayRAM (2, 4, buffer); // Variante LTC1799
		}		
		else
		{
			result *= 256;					// Prescaler 74LVC74 -> LTC6905 + Timer 0 Prescaler 64:1
			result /= 1000;					// scale result -> LTC6905
			ltoa_format (buffer, result, 3, 2, ' '); // format display output LTC6905
			LCD_displayRAM (2, 5, buffer); // Variante LTC6905
		}
									
		LCD_displayROM (2, 11, " MHz");					
		
		ConvertADC();         // Start conversion
		while( BusyADC() );   // Wait for completion
		result = ReadADC();   // Read result
		
		result *= 4000;
		result /= 2457;

		ltoa_format (buffer, result, 3, 1, ' ');
		
		LCD_displayRAM (1, 6, buffer);					
		LCD_displayROM (1, 11, " mV");				
		
		PIR1bits.TMR1IF	= 0;							// clear Timer1 interrupt flag bit
		PIE1bits.TMR1IE = 1;							// enable Timer1 overflow interrupts 
		
		WriteTimer1 (34285);							// load timer 1: 34285 -> 10ms time base, fq=10 MHz	
		//WriteTimer1 (46784);							// load timer 1: 46784 -> 10ms time base, fq=6 MHz		
		
		WriteTimer0 (0);								// clear Timer 0 register
		
		T1CONbits.TMR1ON = 1;							// enable Timer 1
		T0CONbits.TMR0ON = 1;							// enable Timer 0
 }

// End of interrupt processing code 

void main (void)
{
       
  	TRISCbits.TRISC4 = 0;							//RC4 --> output
  	TRISAbits.TRISA4 = 1;							//RA4 --> input
  
  	RCONbits.IPEN = 1;								// enable prioritized interrupts
	IPR1bits.TMR1IP = 1;							// timer 1 high priority
	PIE1bits.TMR1IE = 1;							// enable timer 1 overflow interrupt
	
	LCD_initialise ();  							// init LCD. (code WILL HANG HERE if no LCD is present)
	
	if (PORTCbits.RC2)
  		LCD_displayROM (1, 1, "Version LTC1799");	// Firmware version V1
  	else 
  		LCD_displayROM (1, 1, "Version LTC6905");	// Firmware version V2
  	LCD_displayROM (2, 1, "K. Riedel,01/11");		// Firmware version
  	
  	SleepMS(2000);
  	LCD_command (0x01);             				// clear display - takes ~16 ms
    SleepMS(20);
  	
  	OpenADC( ADC_FOSC_8 & ADC_RIGHT_JUST & ADC_4_TAD, ADC_CH1 & ADC_REF_VDD_VSS &
           ADC_INT_OFF, ADC_2ANA );
    	
  	WriteTimer1 (34285);							// load timer 1: 34285 -> 10ms time base, fq=10 MHz
  	//WriteTimer1 (46784);							// load timer 1: 46784 -> 10ms time base, fq=6 MHz
	
	OpenTimer1( TIMER_INT_ON & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_8 & T1_OSC1EN_OFF &
	            T1_SYNC_EXT_OFF );

	OpenTimer0( TIMER_INT_OFF & T0_16BIT &  T0_SOURCE_EXT &  T0_PS_1_64 );

 	INTCONbits.GIE = 1;								// enable global interrupts
	 
  while (1)
  {
	//idle loop
	}
}
